<?php

namespace Drupal\theme_by_author;

use Drupal\Core\Routing\RouteMatchInterface;
use Drupal\Core\Theme\ThemeNegotiatorInterface;
use Drupal\user\EntityOwnerInterface;

class AuthorThemeNegotiator implements ThemeNegotiatorInterface  {

  /**
   * @inheritDoc
   */
  public function applies(RouteMatchInterface $route_match) {
    $author = $this->getAuthorFromRouteMatch($route_match);
    return !empty($author) && !empty($author->theme->value);
  }

  /**
   * @inheritDoc
   */
  public function determineActiveTheme(RouteMatchInterface $route_match) {
    $author = $this->getAuthorFromRouteMatch($route_match);
    return !empty($author->theme->value) ? $author->theme->value : NULL;
  }

  /**
   * Returns the entity owner's user entity, if the given route matches a
   * canonical entity route, and that entity has an owner.
   *
   * @param \Drupal\Core\Routing\RouteMatchInterface $route_match
   *   The current route match object.
   * @return \Drupal\user\UserInterface|null
   *   The author of the given route's object. It must be a canonical entity
   *   route and the entity must implement \Drupal\user\EntityOwnerInterface
   *   in order to be able to determine an author at all.
   */
  protected function getAuthorFromRouteMatch(RouteMatchInterface $route_match) {
    $canonical_entity_route_pattern = '/entity\.(\w+)\.canonical/';
    $matches = [];
    preg_match($canonical_entity_route_pattern, $route_match->getRouteName(), $matches);
    if (!empty($matches)) {
      $entity_type = $matches[1];
      $entity = $route_match->getParameter($entity_type);
      if (!empty($entity) && $entity instanceof EntityOwnerInterface) {
        return $entity->getOwner();
      }
    }
    return NULL;
  }
}